package sdanderson.androidcharts.Common;

import android.content.Context;
import android.graphics.Canvas;
import android.text.TextPaint;
import android.util.AttributeSet;
import android.view.Gravity;
import android.widget.TextView;

/**
 * Created by Shane Anderson on 10/2/2015.
 *
 */
public class VerticalTextView extends TextView {
    final boolean topDown;

    public VerticalTextView(Context pContext, AttributeSet pAttrs){
        super(pContext, pAttrs);
        final int gravity = getGravity();
        if(Gravity.isVertical(gravity) &&
                (gravity&Gravity.VERTICAL_GRAVITY_MASK) == Gravity.BOTTOM) {
            setGravity((gravity&Gravity.HORIZONTAL_GRAVITY_MASK) | Gravity.TOP);
            topDown = true;
        }else
            topDown = false;
    }

    @Override
    protected void onMeasure(int pWidthMeasureSpec, int pHeightMeasureSpec){
        super.onMeasure(pHeightMeasureSpec, pWidthMeasureSpec);
        setMeasuredDimension(getMeasuredHeight(), getMeasuredWidth());
    }

    @Override
    protected void onDraw(Canvas pCanvas){
        TextPaint textPaint = getPaint();
        textPaint.setColor(getCurrentTextColor());
        textPaint.drawableState = getDrawableState();

        pCanvas.save();

        if(topDown){
            pCanvas.translate(getWidth(), 0);
            pCanvas.rotate(90);
        }else {
            pCanvas.translate(0, getHeight());
            pCanvas.rotate(-90);
        }


        pCanvas.translate(getCompoundPaddingLeft(), getExtendedPaddingTop());

        getLayout().draw(pCanvas);
        pCanvas.restore();
    }
}