package sdanderson.androidcharts.PieChart;

import android.content.Context;
import android.content.res.Resources;
import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.RectF;
import android.graphics.drawable.BitmapDrawable;
import android.util.AttributeSet;
import android.util.TypedValue;
import android.view.Gravity;
import android.view.LayoutInflater;
import android.view.MotionEvent;
import android.view.View;
import android.view.ViewGroup;
import android.view.WindowManager;
import android.widget.PopupWindow;
import android.widget.TextView;

import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.List;

import sdanderson.androidcharts.R;
import sdanderson.androidcharts.Common.CoreSettings;
import sdanderson.androidcharts.Common.NumberUtilities;

/**
 * Created by Shane Anderson on 10/18/2014.
 *
 */
public class PieChart extends View implements PieChartItem.OnFinishedAnimation {

    private float mBorderWidth = 5f;
    private int mBorderColor = Color.BLACK;
    private boolean mPercentalize = false;
    DecimalFormat mDecimalFormatPopupValue = new DecimalFormat("#.##");
    private boolean mUseNumberAbbreviationSuffixValue = false;
    private String mNumberSuffixes = "kmbt";
    private int mNumberGroupSplit = 3;

    private RectF mRectF = new RectF();

    PopupWindow mLingeringPopupWindow = null;

    //private PieChartInnerView mPieChartInnerView;

    private OnSliceTouchedListener mOnSliceTouchedListener = null;
    private List<PieChartItem> mSlice;
    private float mTotal = 0.0f;
    private Paint mDrawingPaint = new Paint();
    private RectF mDrawingRect = new RectF();

    Paint mBorderPaint = new Paint();

    // Animation Variables
    private int mAnimFinishedSlices = 0;
    private float mAnimationTime = 500f; // in milliseconds
    private long mAnimationElapsedTime = 0;
    private long mAnimationStartTime = 0;
    private boolean mAnimationCounterStarted = false;

    private Runnable mRedrawRunnable = new Runnable() {
        public void run() {
            try {
                if (mAnimFinishedSlices != mSlice.size()) {
                    if (!mAnimationCounterStarted) {
                        mAnimationStartTime = System.currentTimeMillis();
                        mAnimationCounterStarted = true;
                    }
                    mAnimationElapsedTime = System.currentTimeMillis() - mAnimationStartTime;
                    invalidate();
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    };

    public PieChart(Context pContext)
    {
        super(pContext);
        mSlice = new ArrayList<>();
        setWillNotDraw(false);
        this.setBackgroundColor(Color.TRANSPARENT);
        init();
    }

    public PieChart(Context pContext, AttributeSet pArguments) {
        super(pContext, pArguments);
        mSlice = new ArrayList<>();
        setWillNotDraw(false);
        this.setBackgroundColor(Color.TRANSPARENT);
        init();
    }

    public PieChart(Context pContext, AttributeSet pArguments, int pDefStyle) {
        super(pContext, pArguments, pDefStyle);
        mSlice = new ArrayList<>();
        setWillNotDraw(false);
        this.setBackgroundColor(Color.TRANSPARENT);
        init();
    }

    private void init()
    {
        Paint piePaint;
        setBackgroundColor(Color.TRANSPARENT);
        piePaint = new Paint(Paint.ANTI_ALIAS_FLAG);
        piePaint.setStyle(Paint.Style.FILL);

        setOnTouchListener(new OnTouchListener() {
            float mX = 0, mY = 0;

            @Override
            public boolean onTouch(View view, MotionEvent motionEvent) {
                if (motionEvent.getAction() == MotionEvent.ACTION_DOWN) {
                    mX = motionEvent.getX();
                    mY = motionEvent.getY();
                }
                if (motionEvent.getEventTime() - motionEvent.getDownTime() < 500 &&
                        motionEvent.getAction() == MotionEvent.ACTION_UP) {
                    if (mOnSliceTouchedListener != null) {
                        mOnSliceTouchedListener.onSliceTouched(tapped(mX, mY),
                                PieChart.this, mX, mY);
                    } else {
                        defaultTouchResponse(tapped(mX, mY), PieChart.this, mX, mY);
                    }
                }
                return true;
            }
        });
    }

    @Override
    public void onDraw(final Canvas pCanvas) {
        if (pCanvas == null)
            return;

        if (mRectF != null && mBorderPaint != null && mRedrawRunnable != null && !isInEditMode()) {
            mRectF.set(5, 5, getWidth() - 5, getHeight() - 5);

            if (mSlice != null && !mSlice.isEmpty()) {
                for (PieChartItem sprite : mSlice) {
                    sprite.draw(pCanvas, mRectF, mAnimationElapsedTime/mAnimationTime);
                }
            }

            // Draw the separation lines and the line around the piechart
            mBorderPaint.setColor(mBorderColor);
            mBorderPaint.setStrokeWidth(mBorderWidth);
            mBorderPaint.setStyle(Paint.Style.STROKE);
            mBorderPaint.setAntiAlias(true);
            pCanvas.drawCircle(
                    mRectF.centerX(), mRectF.centerY(), mRectF.width() / 2, mBorderPaint);

            if (mSlice != null && !mSlice.isEmpty()) {
                for (PieChartItem sprite : mSlice) {
                    // Need the x/y coord on the outer edge of the circle
                    float outx = mRectF.centerX() + (float)
                            Math.cos(Math.toRadians(sprite.mStartAngle)) * mRectF.width() / 2;
                    float outy = mRectF.centerY() + (float)
                            Math.sin(Math.toRadians(sprite.mStartAngle)) * mRectF.height() / 2;

                    pCanvas.drawLine(mRectF.centerX(), mRectF.centerY(), outx, outy, mBorderPaint);
                }
            }

            postDelayed(mRedrawRunnable, CoreSettings.ANIMATION_RUNNABLE_RESOLUTION);
        } else if (isInEditMode()) {
            // The default view which is used in the layout preview
            mDrawingRect.set(5, 5, getWidth() - 5, getHeight() - 5);
            mDrawingPaint.setColor(Color.BLACK);
            mDrawingPaint.setStrokeWidth(2.0f);
            mDrawingPaint.setStyle(Paint.Style.STROKE);
            mDrawingPaint.setAntiAlias(true);
            pCanvas.drawCircle(mDrawingRect.centerX(), mDrawingRect.centerY(),
                    mDrawingRect.width() / 2, mDrawingPaint);
        }
    }

    @Override
    protected void onLayout(boolean pChanged, int pL, int pT, int pR, int pB) {
        super.onLayout(pChanged, pL, pT, pR, pB);
    }

    @Override
    protected void onSizeChanged(int pW, int pH, int pOldW, int pOldH) {
        super.onSizeChanged(pW, pH, pOldW, pOldH);
        dataChanged();
    }

    @Override
    public void onSliceFinishedAnimation() { mAnimFinishedSlices++; }

    @Override
    protected void onDetachedFromWindow() {
        super.onDetachedFromWindow();
        if (mLingeringPopupWindow != null && mLingeringPopupWindow.isShowing()) {
            mLingeringPopupWindow.dismiss();
        }
    }

    protected void dataChanged() {
        float currentAngle = (float) Math.random()*360f;
        for (PieChartItem it : mSlice) {
            it.mStartAngle = currentAngle;
            it.mEndAngle = it.mStartAngle + it.mValue * 360.0f / mTotal;
            it.mMiddleAngle = (it.mStartAngle + it.mEndAngle)/2f;
            it.mCurrentStartAngle = it.mStartAngle;
            it.mCurrentEndAngle = it.mStartAngle +0.5f;
            currentAngle = it.mEndAngle;
        }
    }

    private void defaultTouchResponse(int pColor, PieChart pPieChart, float pTouchedX,
                                      float pTouchedY) {
        PieChartItem slice = pPieChart.getSliceByColor(pColor);

        // If null, we clicked on the border or some other area of the pie chart that doesn't
        // represent a pie chart slice
        if (slice != null) {
            LayoutInflater inflater = (LayoutInflater)
                                     getContext().getSystemService(Context.LAYOUT_INFLATER_SERVICE);
            View v = inflater.inflate(R.layout.piebarchart_popupwindow,
                                                    (ViewGroup) getParent(), false);
            TextView tv = (TextView) v.findViewById(R.id.piebarchart_popupwindow_slicetitletext);
            tv.setText(slice.getLabel());
            tv = (TextView) v.findViewById(R.id.piebarchart_popupwindow_slicetitlevalue);

            if (isPercentalized()) {
                Float tmp = (slice.getValue())*100.0f/getPiechartTotal();
                String txtOut;
                // Should never have anything that needs to be abbreviated in most number systems,
                // but just in case there is some weird abbreviation system where it divides every
                // second place, let people mix percentalization and suffixization
                if (mUseNumberAbbreviationSuffixValue)
                    txtOut = NumberUtilities.formatValueWithSuffix(tmp,
                            mDecimalFormatPopupValue,
                            mNumberSuffixes,
                            mNumberGroupSplit)+"%";
                else
                    txtOut = mDecimalFormatPopupValue.format(tmp)+'%';
                tv.setText(txtOut);
            }
            else {
                if (mUseNumberAbbreviationSuffixValue)
                    tv.setText(NumberUtilities.formatValueWithSuffix(slice.getValue(),
                            mDecimalFormatPopupValue,
                            mNumberSuffixes,
                            mNumberGroupSplit));
                else
                    tv.setText(mDecimalFormatPopupValue.format(slice.getValue()));
            }

            View colorkey = v.findViewById(R.id.piebarchart_popupwindow_colorkey);
            colorkey.setBackgroundColor(pColor);
            final PopupWindow pw = new PopupWindow(v, WindowManager.LayoutParams.WRAP_CONTENT,
                                                        WindowManager.LayoutParams.WRAP_CONTENT);
            mLingeringPopupWindow = pw;
            // Need to set a background drawable to some random bitmap to make it touchable (bug?)
            pw.setBackgroundDrawable(new BitmapDrawable(getContext().getResources(),
                    Bitmap.createBitmap(1, 1, Bitmap.Config.ARGB_8888)));
            pw.setOutsideTouchable(true);
            pw.setTouchInterceptor(new View.OnTouchListener() {
                @Override
                public boolean onTouch(View view, MotionEvent motionEvent) {
                    pw.dismiss();
                    return true;
                }
            });
            View parentView = (View) getParent();
            int[] location = new int[2];
            pPieChart.getLocationOnScreen(location);
            Resources r = getResources();
            float pxY   = TypedValue.applyDimension(
                                        TypedValue.COMPLEX_UNIT_DIP, 80, r.getDisplayMetrics());
            pw.showAtLocation(parentView, Gravity.NO_GRAVITY,
                    location[0] + (int) pTouchedX, location[1] + (int) pTouchedY - (int) (pxY));
        }
    }

    protected float getPiechartTotal() {
        float total = 0;
        for (PieChartItem i : mSlice) {
            total += i.getValue();
        }

        return total;
    }

    protected int tapped(float pX, float pY) {
        Bitmap b = Bitmap.createBitmap(getWidth(), getHeight(), Bitmap.Config.ARGB_8888);
        Canvas c = new Canvas(b);
        this.layout(getLeft(), getTop(),  getRight(), getBottom());
        this.draw(c);
        return b.getPixel((int)pX, (int)pY);
    }

    // PUBLICLY AVAILABLE INTERACTION METHODS

    @SuppressWarnings("unused")
    public void addSlice(String pLabel, float pValue, int pColor) {
        PieChartItem pieChart = new PieChartItem();
        pieChart.mLabel = pLabel;
        pieChart.mValue = pValue;
        pieChart.setPieSliceColor(pColor);
        pieChart.setFinishedAnimationHandler(this);

        mTotal += pValue;

        mSlice.add(pieChart);
    }

    @SuppressWarnings("unused")
    public PieChartItem getSliceByColor(int pColor) {
        for (PieChartItem item : mSlice) {
            if (item.getColor() == pColor)
                return item;
        }
        return null;
    }

    @SuppressWarnings("unused")
    public void percentalize(boolean pPercentalize) {
        mPercentalize = pPercentalize;
    }

    @SuppressWarnings("unused")
    public boolean isPercentalized() {
        return mPercentalize;
    }

    @SuppressWarnings("unused")
    public void setBorderWidth(float pBorderWidth) {
        mBorderWidth = pBorderWidth;
    }

    @SuppressWarnings("unused")
    public void setBorderColor(int pBorderColor) {
        mBorderColor = pBorderColor;
    }

    @SuppressWarnings("unused")
    public void setDecimalFormatPopupValue (DecimalFormat pDecimalFormat) {
        mDecimalFormatPopupValue = pDecimalFormat;
    }

    @SuppressWarnings("unused")
    public void setUseNumberAbbreviationSuffixValue(boolean pUseSuffix) {
        mUseNumberAbbreviationSuffixValue = pUseSuffix;
    }

    @SuppressWarnings("unused")
    public void setNumberSuffixes (String pSuffixes) { mNumberSuffixes = pSuffixes; }

    @SuppressWarnings("unused")
    public void setNumberGroupSplit (int pGroupSplit) { mNumberGroupSplit = pGroupSplit; }

    @SuppressWarnings("unused")
    public void setUseAnimation (boolean pUseAnimation) {
        for (PieChartItem item : mSlice) {
            item.setUseAnimation(pUseAnimation);
        }
    }

    @SuppressWarnings("unused")
    public void setAnimationTime(float pAnimationTime) { mAnimationTime = pAnimationTime; }

    @SuppressWarnings("unused")
    public void reanimate() {
        mAnimFinishedSlices = 0;
        for (PieChartItem sprite : mSlice) {
            sprite.reanimate();
        }
        mAnimationElapsedTime = 0;
        mAnimationStartTime = 0;
        mAnimationCounterStarted = false;
        invalidate();
    }

    @SuppressWarnings("unused")
    public void setOnSliceTouchedListener(OnSliceTouchedListener pOnSliceTouchedListener) {
        mOnSliceTouchedListener = pOnSliceTouchedListener;
    }

    @SuppressWarnings("unused")
    public interface OnSliceTouchedListener {
        void onSliceTouched(int pColor, PieChart pPieChart, float pTouchedX, float pTouchedY);
    }
}